@@ -10,14 +10,27 @@ import pandas as pd
 
 '''
 
-'''
-Ishigami test function accepting three parameters in an array_like object as ``x``
-'''
 def ishigami(
         x,
         a: float=7,
         b: float=0.05
 ) -> float:
+    """Ishigami test function.
+
+    Parameters
+    ----------
+    x : array_like
+        a list of parameters
+    a : float, optional
+    b : float, optional
+
+
+    Returns
+    -------
+    model_val : float
+        float value representing Ishigami calculation
+
+    """
     # check whether the input x is an array-like
 
     if not isinstance(x, (pd.core.frame.DataFrame, pd.core.series.Series, np.ndarray, list)):
@@ -33,10 +46,21 @@ def ishigami(
     return np.sin(x[0]) + a * (np.sin(x[1])**2) + b * (x[2]**4) * np.sin(x[0])
 
 
-'''
-Linear additive test function accepting two parameters in an array_like object as ``x``
-'''
 def linear_additive(x):
+    """Linear additive test function accepting two parameters in an array_like object as ``x``
+
+    Parameters
+    ----------
+    x : array_like
+        a list of parameters
+
+
+    Returns
+    -------
+    model_val : float
+        float value representing linear additive calculation
+
+    """
     term1 = 2*x[0]
     term2 = 3*x[1]
     return term1 + term2
\ No newline at end of file
