@@ -16,7 +16,7 @@ from typing import (
     List,
     Tuple,
     Dict,
-    Union
+    Union, Optional
 )
 
 from tqdm.auto import tqdm
@@ -196,7 +196,7 @@ def rn2rx(distpair_type: List,
         rx = newton_krylov(F=fun, xin=rnpair, x_tol=1e-5)
     except:
         print("Function could not converge, fictive matrix was not computed")
-        print("It is recommended to switch corr_flag parameter to True to assume that correlation "
+        print("It is recommended to switch fictive_mat_flag parameter to False to assume that correlation "
               "matrix is equal to fictive matrix")
         rx = rnpair
 
@@ -255,7 +255,8 @@ def map_2_cornorm(parameters: Dict[Union[str, int], Tuple[Union[float, str]]],
 
 
 def n2x_transform(norm_vectors: np.ndarray,
-                  param_info: List
+                  parameters: Dict,
+                  dist_sample_file: Optional[str] = None
                   ) -> np.ndarray:
     """
     transforms multivariate normal samples into parameters original distributions
@@ -264,8 +265,10 @@ def n2x_transform(norm_vectors: np.ndarray,
     ----------
     norm_vectors : np.ndarray
         multivariate normal samples
-    param_info : list
-        a list containing parameter information (bounds, distributions, etc.)
+    parameters : dict
+        a dictionary containing parameter information (name: bounds, distributions, etc.)
+    dist_sample_file : String
+        name of file that contains custom distribution data, optional only for users with custom distributions
 
     Returns
     -------
@@ -288,24 +291,24 @@ def n2x_transform(norm_vectors: np.ndarray,
 
     # Transform from correlated standard normal to original distributions
 
-    k = norm_vectors.shape[1]
     x = np.zeros(norm_vectors.shape)
+    i = 0
 
-    for i in range(0, k):
-        if param_info[i][3] == 'unif':
-            lb = param_info[i][0]
-            ub = param_info[i][1]
+    for param in parameters.keys():
+        if parameters[param][3] == 'unif':
+            lb = parameters[param][0]
+            ub = parameters[param][1]
 
             x[:, i] = lb + (ub - lb) * stat.norm.cdf(norm_vectors[:, i], 0, 1)
-        elif param_info[i][3] == 'norm':
-            mu = param_info[i][0]
-            std = param_info[i][1]
+        elif parameters[param][3] == 'norm':
+            mu = parameters[param][0]
+            std = parameters[param][1]
 
             x[:, i] = stat.norm.ppf(stat.norm.cdf(norm_vectors[:, i], 0, 1), mu, std)
-        elif param_info[i][3] == 'triangle':
-            a = param_info[i][0]
-            b = param_info[i][1]
-            c = param_info[i][2]
+        elif parameters[param][3] == 'triangle':
+            a = parameters[param][0]
+            b = parameters[param][1]
+            c = parameters[param][2]
             mid = (c - a) / (b - a)
             term1 = (b - a) * (c - a)
             term2 = (b - a) * (b - c)
@@ -313,21 +316,24 @@ def n2x_transform(norm_vectors: np.ndarray,
             x[:, i] = (a + np.sqrt(term1) * np.sqrt(x_norm)) * ((x_norm >= 0).astype(int)) * (
                 (x_norm < mid).astype(int)) + (b - np.sqrt(term2) * np.sqrt((1 - x_norm))) * (
                           (x_norm >= mid).astype(int)) * ((x_norm < 1).astype(int))
-        elif param_info[i][3] == 'lognorm':
-            mu = param_info[i][0]
-            std = param_info[i][1]
+        elif parameters[param][3] == 'lognorm':
+            mu = parameters[param][0]
+            std = parameters[param][1]
             term1 = std / mu ** 2
-            m = np.log(mu / (np.sqrt(1 + term1)))
-            v = np.sqrt(np.log(1 + term1))
-            x[:, i] = stat.lognorm.ppf(stat.norm.cdf(norm_vectors[:, i], 0, 1), scale=np.exp(mu), s=std, loc=0)
-        elif param_info[i][3] == 'expo':
-            mu = param_info[i][0]
+            x[:, i] = np.lognorm.ppf(stat.norm.cdf(norm_vectors[:, i], 0, 1), scale=np.exp(mu), s=std, loc=0)
+        elif parameters[param][3] == 'expo':
+            mu = parameters[param][0]
             x[:, i] = np.expon.ppf(stat.norm.cdf(norm_vectors[:, i], 0, 1), scale=mu)
-        elif param_info[i][3] == 'gev':
-            mu = param_info[i][0]  # location
-            sigma = param_info[i][1]  # scale
-            k = -1 * param_info[i][2]  # shape
+        elif parameters[param][3] == 'gev':
+            mu = parameters[param][0]  # location
+            sigma = parameters[param][1]  # scale
+            k = -1 * parameters[param][2]  # shape
             x[:, i] = stat.genextreme.ppf(stat.norm.cdf(norm_vectors[:, i], 0, 1), c=k, scale=sigma, loc=mu)
+        elif parameters[param][3] == 'custom':
+            cdp = custom_distribution_probabilites(dist_sample_file, param)
+            x[:, i] = np.interp(stat.norm.cdf(norm_vectors[:, i], 0, 1), cdp['Probabilities'], cdp[param])
+
+        i += 1
 
     return x
 
@@ -380,7 +386,7 @@ def reorder_pairs(pair_df: pd.DataFrame,
     parameters : dictionary
         dictionary containing parameter names and their attributes
     df : pd.DataFrame
-        Pandas DataFrame containing the star points, and model outputs
+        Pandas DataFrame containing the star points
     delta_h : float
         resolution of star samples
     report_verbose : boolean
@@ -469,7 +475,7 @@ def reorder_pairs(pair_df: pd.DataFrame,
     return pair_df
 
 
-def find_boundaries(parameters):
+def find_boundaries(parameters, dist_sample_file: Optional[str] = None):
     """
     finds maximum and minimum boundary of each parameter.
 
@@ -477,6 +483,8 @@ def find_boundaries(parameters):
     ----------
     parameters : Dictionary
         dictionary containing parameters names and attributes
+    dist_sample_file : str
+        name of file containing distributions data
 
     Returns
     -------
@@ -485,30 +493,70 @@ def find_boundaries(parameters):
     xmax : array_like
         the upper boundaries of each parameter
     """
-    # store parameter info in a list
-    param_info = list(parameters.values())
+
+    # store distributions in dataframe
+    distributions_df = pd.DataFrame()
+    if dist_sample_file:
+        distributions_df = pd.read_csv(dist_sample_file)
 
     # store the max and min values of each paramter in arrays
     xmin = np.zeros(len(parameters))
     xmax = np.zeros(len(parameters))
-    for i in range(0, len(parameters)):
-        if param_info[i][3] == 'unif':
-            xmin[i] = param_info[i][0]  # lower bound
-            xmax[i] = param_info[i][1]  # upper bound
-        elif param_info[i][3] == 'triangle':
-            xmin[i] = param_info[i][0]  # lower bound
-            xmax[i] = param_info[i][1]  # upper bound
-        elif param_info[i][3] == 'norm':
-            xmin[i] = param_info[i][0] - 3 * param_info[i][1]
-            xmax[i] = param_info[i][0] + 3 * param_info[i][1]
-        elif param_info[i][3] == 'lognorm':
-            xmin[i] = 1
-            xmax[i] = 1.25
-        elif param_info[i][3] == 'expo':
-            xmin[i] = 0  # change this
-            xmax[i] = 0  # change this
-        elif param_info[i][3] == 'gev':
-            xmin[i] = 0  # change this
-            xmax[i] = 0  # change this
+    index = 0
+    for param in parameters.keys():
+        if parameters[param][3] == 'unif':
+            xmin[index] = parameters[param][0]  # lower bound
+            xmax[index] = parameters[param][1]  # upper bound
+        elif parameters[param][3] == 'triangle':
+            xmin[index] = parameters[param][0]  # lower bound
+            xmax[index] = parameters[param][1]  # upper bound
+        elif parameters[param][3] == 'norm':
+            xmin[index] = parameters[param][0] - 3 * parameters[param][1]
+            xmax[index] = parameters[param][0] + 3 * parameters[param][1]
+        elif parameters[param][3] == 'lognorm':
+            xmin[index] = 1
+            xmax[index] = 1.25
+        elif parameters[param][3] == 'expo':
+            xmin[index] = 0  # change this
+            xmax[index] = 0  # change this
+        elif parameters[param][3] == 'gev':
+            xmin[index] = 0  # change this
+            xmax[index] = 0  # change this
+        elif parameters[param][3] == 'custom':
+            # read in file and find min and max
+            xmin[index] = distributions_df[param].min()
+            xmax[index] = distributions_df[param].max()
+        index += 1
 
-    return xmin, xmax
\ No newline at end of file
+    return xmin, xmax
+
+
+def custom_distribution_probabilites(dist_sample_file: Optional[str], param):
+    """
+    finds empirical cdf for custom probability distribution and puts it in a dataframe.
+
+    Parameters
+    ----------
+    dist_sample_file : str
+        string name of .csv file containing custom distribution data
+    param : String
+        name of parameter
+
+    Returns
+    -------
+    cdp : array_like
+        df containing custom distributions and empirical cdf
+    """
+
+    cdp = pd.read_csv(dist_sample_file)
+
+    # get just the singular parameter distribution
+    cdp = cdp[param].to_frame().dropna(how='all').reset_index(drop=True)
+
+    # sort data from smallest to largest
+    cdp.sort_values(by=param, ignore_index=True, inplace=True)
+
+    # find empirical cdf for all parameters
+    cdp['Probabilities'] = (cdp.index + 1) / (cdp.shape[0] + 1)
+
+    return cdp
