@@ -425,7 +425,7 @@ def ivars(
 
     # calculate interpolated values for both x (h) and y (variogram)
     if x_int[-1] < scale:
-        x_int.append(scale)
+        x_int = np.append(x_int, scale)
     y_bench = [0] + variogram_array.to_list()
 
     y_int = np.interp(x=x_int, xp=x_bench, fp=y_bench)
@@ -865,7 +865,7 @@ def bootstrapping(
 
         bootstrapped_ecovariogram = e_covariogram(bootstrapped_cov_section_all)
 
-        bootstrapped_var = bootstrapped_df.iloc[:, -1].unique().var(ddof=1)
+        bootstrapped_var = np.nanvar(bootstrapped_df.iloc[:, -1].unique(), ddof=1)
 
         bootstrapped_sobol = sobol_eq(bootstrapped_variogram, bootstrapped_ecovariogram,
                                       bootstrapped_var, delta_h)
